/*
  acpi.h: structures and declaration of functions to access /proc/acpi
  (C) 2002 Oliver Diedrich

  This file may be redistributed under the terms of the
  GNU Public License (GPL), see www.gnu.org
*/

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#define VERSION "acpi watch 0.75"
#define PROC "/proc/acpi"


/**************************************/
/* structures used in /proc/acpi tree */
/**************************************/

typedef struct _file file;
struct _file
/* one directory or file in /proc/acpi */
{
  char *name;       /* name of directory or file */
  mode_t mode;      /* file type and access rights as st_mode in stat(2) */
                    /* structure: directory or file, rwx */
  void *data;       /* pointer to 1st file in directory or to file content */
  int changed;      /* 1: at least one file in dir / one entry in file has */
                    /* changed */
  file *next;       /* next file in the recent directory */
  void *iter;       /* in GtkTree when built with GUI */
};


typedef struct _line line;
struct _line
/* one line in a text file */
{
  char *label;      /* label of entry */
  char *value;      /* value, separated from label by re "\: +" */
                    /* NULL in entries without label/value distinction */
                    /* binary data for binary files, starting with " \0\0\0" */
                    /* followed by a 4 byte number with the number of bytes */
                    /* including the 8 header bytes */
  time_t time;      /* seconds since the epoch when this line was read */
  line *newer;      /* chronological list of newer versions of line */
  line *latest;     /* points to latest entry in newer list */
  int changed;      /* 1: entry has changed */
  line *next;       /* next line */
  void *iter;       /* in GtkTree when built with GUI */
};


typedef struct _sysinfo sysinfo;
struct _sysinfo
{
/* from /proc/cpuinfo and /proc/meminfo */
  char *cpu;
  int mhz;
  unsigned int mem;
/* from uname(2) */
  char *sysname;
  char *release;
  char *version;
  char *machine;
/* DMI info */
  char *sys_vendor;
  char *sys_product;
  char *sys_serial;
  char *bios_vendor;
  char *bios_version;
  char *bios_release;
  char *board_vendor;
  char *board_product;
  char *board_version;
  char *board_serial;
  char *cpu_maker;
  char *cpu_family;
  char *cpu_version;
  char *chassis;
};




/********************************/
/* functions defined in acpiw.c */
/********************************/

void usage(char *myname);
line *readfile(char *name, long seconds);
file *build_tree(char *dirname);
int rescan_tree(file *tree, char *basedir);
file *new_file(char *name);
line *new_line(long seconds, char *label, char *value);
int get_acpi_version(char *procname, char *version);
int get_acpi_info(char ***acpi_info);
int save_all(file *acpi_tree, FILE *f, char *basedir, char **acpi_info, 
             int n_acpi_info, char acpi_version[], sysinfo si);

/**********************************/
/* functions defined in sysinfo.c */
/**********************************/

int get_dmi_info(sysinfo *si);
void get_basic_sysinfo(sysinfo *si);
