/*
  menu.c: menu bar for acpiw GUI, including menu callback functions
  (C) 2002 Oliver Diedrich

  This file may be redistributed under the terms of the
  GNU Public License (GPL), see www.gnu.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <assert.h>
#include "acpiw.h"

extern line *selected;        /* line just shown in text window */
extern char *fname;           /* file to write ACPI data to */
extern file *acpi_tree;       /* internal ACPI tree */
extern char **acpi_info;      /* ACPI info read from syslog */
extern int n_acpi_info;       /* # lines in acpi_info */
extern char acpi_version[];   /* from /proc/acpi/info */
extern sysinfo si;            /* struct with system information */


static void on_about(GtkMenuItem *i, GtkWindow *main_window)
/* displays the about window */
{
  GtkWidget *dialog, *label;
  char str[256];
  
  dialog = gtk_dialog_new_with_buttons ("About",
                                        main_window,
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_STOCK_OK,
                                        GTK_RESPONSE_NONE,
                                        NULL);
  sprintf(str, "\n\n  <b>%s</b>  \n\n  (C) 2002 Oliver Diedrich  \n\n",
          VERSION);
  label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), str);
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
  gtk_widget_show_all(label);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
} /* on_about() */



static void on_acpi_info(GtkMenuItem *menu, GtkTextBuffer *text)
/* displays ACPI info in text view */
{
  GtkTextIter it;        /* needed to write to text */
  int i;
  char **c;
  char str[1024];

  gtk_text_buffer_set_text(text, "\0", -1);
  gtk_text_buffer_get_iter_at_offset(text, &it, 0);
  selected = NULL;  /* no ACPI tree item is displayed that must be updated */
  gtk_text_buffer_insert_with_tags_by_name(text, &it, 
                                           "ACPI related syslog messages\n\n",
                                           -1, "bold", 0);
  c = acpi_info;
  for (i=0; i<n_acpi_info; i++, c++)
  {
    sprintf(str, "%s\n", *c);
    gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);
  }
} /* on_acpi_info() */




static void on_system_info(GtkMenuItem *menu, GtkTextBuffer *text)
/* displays system information in text view */
{
  GtkTextIter it;        /* needed to write to text */
  char str[1024];

  gtk_text_buffer_set_text(text, "\0", -1);
  gtk_text_buffer_get_iter_at_offset(text, &it, 0);
  selected = NULL;  /* no ACPI tree item is displayed that must be updated */
  gtk_text_buffer_insert_with_tags_by_name(text, &it, "System\n", -1, 
                                           "bold", 0);
  sprintf(str, "%s %s (%s)\nserial #: %s\n", si.sys_vendor, si.sys_product,
                                             si.chassis, si.sys_serial);
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);

  gtk_text_buffer_insert_with_tags_by_name(text, &it, "\nBIOS\n", -1,
                                           "bold", 0);
  sprintf(str, "%s %s (%s)\n", si.bios_vendor, si.bios_version,
                               si.bios_release);
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);

  gtk_text_buffer_insert_with_tags_by_name(text, &it, "\nBoard\n", -1, 
                                           "bold", 0);
  sprintf(str, "%s %s (%s)\nserial #: %s\n", si.board_vendor, si.board_product,
                               si.board_version, si.board_serial);
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);

  gtk_text_buffer_insert_with_tags_by_name(text, &it, "\nCPU\n", -1, "bold", 0);
  sprintf(str, "%s %s, %d MHz\n", si.cpu_maker, si.cpu, si.mhz);
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);

  gtk_text_buffer_insert_with_tags_by_name(text, &it, "\nRAM\n", -1, "bold", 0);
  sprintf(str, "%d MByte\n", si.mem / (1024*1023));
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);

  gtk_text_buffer_insert_with_tags_by_name(text, &it, "\nKernel\n", -1, "bold", 0);
  sprintf(str, "%s %s for %s (build %s)\n", si.sysname, si.release, si.machine, 
                                  si.version);
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);
  sprintf(str, "ACPI release %s\n", acpi_version ? acpi_version : "");
  gtk_text_buffer_insert_with_tags_by_name(text, &it, str, -1, 0);
} /* on_system_info() */




static void on_expand(GtkMenuItem *i, GtkTreeView *t)
{
  static int expanded = 0;    /* 1: tree completely expanded */

  if (expanded)
  {
    gtk_tree_view_collapse_all(t);
    expanded = 0;
  }
  else
  {
    gtk_tree_view_expand_all(t);
    expanded = 1;
  }
} /* on_expand() */




void get_filename (GtkWidget *ok_button, GtkFileSelection *file_selector)
{
   fname = (char *) gtk_file_selection_get_filename(file_selector);
}



static void on_save(GtkMenuItem *i, GtkWindow *main_window)
{
  char str[255];
  GtkWidget *dialog;
  FILE *f = NULL;

  if (!fname)
  {
    GtkWidget *file_selector = gtk_file_selection_new("Please select a file");
    g_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
                     "clicked", G_CALLBACK (get_filename),
                     GTK_FILE_SELECTION(file_selector));
    gtk_dialog_run(GTK_DIALOG(file_selector));
    if (file_selector && GTK_IS_WIDGET(file_selector))
     gtk_widget_destroy(file_selector);

    if ( fname && (strlen(fname) > 0) && ('/' == fname[strlen(fname)-1]) )
    {
      free(fname);
      fname = NULL;
      return;
    }
  }
  if (fname) f = fopen(fname, "w");
  else return;
  if (!f) 
  {
    sprintf(str, "can't open file '%s'", fname);
    dialog = gtk_message_dialog_new(main_window,
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    str);
    gtk_dialog_run (GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
  }
  else
  {
    if (save_all(acpi_tree, f, PROC, acpi_info, n_acpi_info, acpi_version, si))
    {
      sprintf(str, "ACPI data written to '%s'", fname);
      dialog = gtk_message_dialog_new(main_window,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_INFO,
                                      GTK_BUTTONS_CLOSE,
                                      str);
    }
    else
    {
      sprintf(str, "error writing ACPI data to '%s'", fname);
      dialog = gtk_message_dialog_new(main_window,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_INFO,
                                      GTK_BUTTONS_CLOSE,
                                      str);
    }
    fclose(f);
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
  }
} /* on_save() */




static void on_save_as(GtkMenuItem *i, GtkWindow *main_window)
{
  char str[255];
  GtkWidget *dialog;
  FILE *f = NULL;

  GtkWidget *file_selector = gtk_file_selection_new("Please select a file");
  g_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
                    "clicked", G_CALLBACK (get_filename), 
                    GTK_FILE_SELECTION(file_selector));
  fname = NULL;     /* user _must_ select a file name, no default file name */
  gtk_dialog_run(GTK_DIALOG(file_selector));
  if ( file_selector && GTK_IS_WIDGET(file_selector))
   gtk_widget_destroy(file_selector);
  if ( fname && (strlen(fname) > 0) && ('/' == fname[strlen(fname)-1]) )
  {
    free(fname);
    fname = NULL;
    return;
  }

  if (fname) f = fopen(fname, "w");
  else return;
  if (!f) 
  {
    sprintf(str, "can't open file '%s'", fname);
    dialog = gtk_message_dialog_new(main_window,
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    str);
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
  }
  else
  {
    if (save_all(acpi_tree, f, PROC, acpi_info, n_acpi_info, acpi_version, si))
    {
      sprintf(str, "ACPI data written to '%s'", fname);
      dialog = gtk_message_dialog_new(main_window,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_INFO,
                                      GTK_BUTTONS_CLOSE,
                                      str);
    }
    else
    {
      sprintf(str, "error writing ACPI data to '%s'", fname);
      dialog = gtk_message_dialog_new(main_window,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_INFO,
                                      GTK_BUTTONS_CLOSE,
                                      str);
    }
    fclose(f);
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
  }
} /* on_save_as() */




GtkWidget *create_menu(GtkWindow *main_window,
                       GtkTreeView *gtk_tree_view,
                       GtkTextBuffer *text,
                       GtkAccelGroup *accel)
/* creates the menu bar with all its callback functions */
/* returns the menu bar */
{
  GtkWidget *menubar,         /* menu */

            *file,            /* main menu item "File" */
            *file_menu,       /* the file menu */
            *save,
            *save_as,
            *separator1,
            *quit,

            *view,            /* main menu item "View" */
            *view_menu,
            *expand,
            *settings,
            *separator2,
            *acpi_info,
            *system_info,

            *help,            /* main menu item "Help" */
            *help_menu,
            *about;

assert(gtk_tree_view != NULL);
assert(text != NULL);

  menubar = gtk_menu_bar_new();
  file = gtk_menu_item_new_with_mnemonic("_File");
  gtk_widget_show(file);
  gtk_container_add(GTK_CONTAINER(menubar), file);

  file_menu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), file_menu);

  save = gtk_menu_item_new_with_mnemonic("_Save");
  gtk_widget_show(save);
  gtk_container_add (GTK_CONTAINER(file_menu), save);
  gtk_widget_add_accelerator (save, "activate", accel, GDK_S,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  g_signal_connect(G_OBJECT(save), "activate",
                   G_CALLBACK(on_save), main_window);

  save_as = gtk_menu_item_new_with_mnemonic("Save _as ...");
  gtk_widget_show(save_as);
  gtk_container_add (GTK_CONTAINER(file_menu), save_as);
  g_signal_connect(G_OBJECT(save_as), "activate",
                   G_CALLBACK(on_save_as), main_window);

  separator1 = gtk_menu_item_new();
  gtk_widget_show(separator1);
  gtk_container_add(GTK_CONTAINER(file_menu), separator1);
  gtk_widget_set_sensitive(separator1, FALSE);

  quit = gtk_menu_item_new_with_mnemonic("_Quit");
  gtk_widget_show(quit);
  gtk_container_add (GTK_CONTAINER(file_menu), quit);
  gtk_widget_add_accelerator (quit, "activate", accel, GDK_Q,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  g_signal_connect(G_OBJECT(quit), "activate",
                   G_CALLBACK(gtk_main_quit), NULL);

  view = gtk_menu_item_new_with_mnemonic("_View");
  gtk_widget_show(view);
  gtk_container_add(GTK_CONTAINER(menubar), view);

  view_menu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(view), view_menu);

  expand = gtk_menu_item_new_with_mnemonic("_Expand/collapse tree");
  gtk_widget_show(expand);
  gtk_container_add (GTK_CONTAINER(view_menu), expand);
  gtk_widget_add_accelerator (expand, "activate", accel, GDK_E,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  g_signal_connect(G_OBJECT(expand), "activate",
                   G_CALLBACK(on_expand), gtk_tree_view);

  settings = gtk_menu_item_new_with_mnemonic("Se_ttings");
  gtk_widget_show(settings);
  gtk_container_add(GTK_CONTAINER(view_menu), settings);
  gtk_widget_add_accelerator (settings, "activate", accel, GDK_T,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  separator2 = gtk_menu_item_new();
  gtk_widget_show(separator2);
  gtk_container_add(GTK_CONTAINER(view_menu), separator2);
  gtk_widget_set_sensitive(separator2, FALSE);

  acpi_info = gtk_menu_item_new_with_mnemonic("_ACPI info");
  gtk_widget_show(acpi_info);
  gtk_container_add (GTK_CONTAINER(view_menu), acpi_info);
  gtk_widget_add_accelerator (acpi_info, "activate", accel, GDK_A,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_object_set_data(GTK_OBJECT(acpi_info), "main_window", main_window);
  g_signal_connect(G_OBJECT(acpi_info), "activate",
                   G_CALLBACK(on_acpi_info), text);

  system_info = gtk_menu_item_new_with_mnemonic("System _info");
  gtk_widget_show(system_info);
  gtk_container_add (GTK_CONTAINER(view_menu), system_info);
  gtk_widget_add_accelerator (system_info, "activate", accel, GDK_I,
                              GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_object_set_data(GTK_OBJECT(system_info), "main_window", main_window);
  g_signal_connect(G_OBJECT(system_info), "activate",
                   G_CALLBACK(on_system_info), text);

  help = gtk_menu_item_new_with_mnemonic("_Help");
  gtk_widget_show(help);
  gtk_container_add(GTK_CONTAINER(menubar), help);

  help_menu = gtk_menu_new();
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(help), help_menu);

  about = gtk_menu_item_new_with_mnemonic("_About");
  gtk_widget_show(about);
  gtk_container_add (GTK_CONTAINER(help_menu), about);
  g_signal_connect(G_OBJECT(about), "activate",
                   G_CALLBACK(on_about), main_window);

  return menubar;
}
